@@ -2,10 +2,13 @@
 
 #![cfg(test)]
 
-use crate::prelude::*;
-
 use tap::conv::TryConv;
 
+use crate::{
+	prelude::*,
+	view::BitViewSized,
+};
+
 #[test]
 fn create_arrays() {
 	macro_rules! make {
@@ -31,7 +34,7 @@ fn create_arrays() {
 fn wrap_unwrap() {
 	let data: [u8; 15] = *b"Saluton, mondo!";
 	let bits = BitArray::<LocalBits, _>::new(data);
-	assert_eq!(bits.value(), data);
+	assert_eq!(bits.into_inner(), data);
 }
 
 #[test]
@@ -75,7 +78,7 @@ fn convert() {
 #[allow(deprecated)]
 fn iter() {
 	let mut iter = bitarr![0, 0, 0, 1, 1, 1, 0, 0, 0].into_iter();
-	let width = <[usize; 1] as BitView>::const_bits();
+	let width = <[usize; 1] as BitViewSized>::BITS;
 
 	let slice = iter.as_slice();
 	let iter_slice = iter.as_bitslice();
