@@ -2,37 +2,94 @@
 
 #![cfg(test)]
 
-use crate::prelude::*;
-
 use core::cell::Cell;
 
+use funty::IsNumber;
+
+use crate::prelude::*;
+
 #[test]
 fn compile_bitarr_typedef() {
+	#[derive(Clone, Copy, Debug, Eq, Ord, PartialEq, PartialOrd)]
 	struct Slots {
-		all: bitarr!(for 10, in Msb0, u8),
-		typ: bitarr!(for 10, in u8),
-		def: bitarr!(for 10),
+		all: BitArr!(for 10, in Msb0, u8),
+		typ: BitArr!(for 10, in u8),
+		def: BitArr!(for 10),
 	}
 
+	static SLOTS: Slots = Slots {
+		all: bitarr!(const Msb0, u8; 1, 1, 1, 1, 1, 1, 1, 1, 1, 1),
+		typ: bitarr!(const Lsb0, u8; 1, 1, 1, 1, 1, 1, 1, 1, 1, 1),
+		def: bitarr!(const           1, 1, 1, 1, 1, 1, 1, 1, 1, 1),
+	};
+
 	let slots = Slots {
 		all: bitarr!(Msb0, u8; 1, 1, 1, 1, 1, 1, 1, 1, 1, 1),
 		typ: bitarr!(Lsb0, u8; 1, 1, 1, 1, 1, 1, 1, 1, 1, 1),
 		def: bitarr!(1, 1, 1, 1, 1, 1, 1, 1, 1, 1),
 	};
 
-	assert_eq!(slots.all.value(), [!0u8, 192]);
-	assert_eq!(slots.typ.value(), [!0u8, 3]);
-	let def: [usize; 1] = slots.def.value();
+	assert_eq!(SLOTS, slots);
+
+	assert_eq!(slots.all.into_inner(), [!0u8, 192]);
+	assert_eq!(slots.typ.into_inner(), [!0u8, 3]);
+	let def: [usize; 1] = slots.def.into_inner();
 	assert_eq!(def[0].count_ones(), 10);
 }
 
+#[test]
+#[allow(clippy::declare_interior_mutable_const)]
+fn constexpr_macros() {
+	const A: BitArr!(for 20, in Lsb0, Cell<u8>) =
+		bitarr!(const Lsb0, Cell<u8>; 1; 20);
+	let a = A;
+	assert_eq!(a.len(), 24);
+	assert!(a.all());
+
+	const B: BitArr!(for 20) = bitarr!(const 1; 20);
+	let b = B;
+	assert_eq!(b.len(), <usize as IsNumber>::BITS as usize);
+	assert!(b.all());
+
+	const C: BitArr!(for 5, in Msb0, Cell<u16>) =
+		bitarr!(const Msb0, Cell<u16>; 1, 0, 1, 1, 0);
+	let c = C;
+	assert_eq!(c[.. 5], bits![1, 0, 1, 1, 0]);
+
+	const D: BitArr!(for 5, in Lsb0, u32) =
+		bitarr!(const Lsb0, u32; 1, 0, 1, 1, 0);
+	let d = D;
+	assert_eq!(d[.. 5], bits![1, 0, 1, 1, 0]);
+
+	let _: &'static mut BitSlice<Msb0, Cell<u16>> =
+		bits!(static mut Msb0, Cell<u16>; 1; 20);
+	let _: &'static mut BitSlice<Lsb0, u32> = bits!(static mut Lsb0, u32; 1; 20);
+	let _: &'static mut BitSlice = bits!(static mut 1; 20);
+
+	let _: &'static mut BitSlice<Msb0, Cell<u16>> =
+		bits!(static mut Msb0, Cell<u16>; 1, 0, 1, 1, 0);
+	let _: &'static mut BitSlice<Lsb0, u32> =
+		bits!(static mut Lsb0, u32; 1, 0, 1, 1, 0);
+	let _: &'static mut BitSlice = bits!(static mut 1, 0, 1, 1, 0);
+
+	let _: &'static BitSlice<Msb0, Cell<u16>> =
+		bits!(static Msb0, Cell<u16>; 1; 20);
+	let _: &'static BitSlice<Lsb0, u32> = bits!(static Lsb0, u32; 1; 20);
+	let _: &'static BitSlice = bits!(static 1; 20);
+
+	let _: &'static BitSlice<Msb0, Cell<u16>> =
+		bits!(static Msb0, Cell<u16>; 1, 0, 1, 1, 0);
+	let _: &'static BitSlice<Lsb0, u32> = bits!(static Lsb0, u32; 1, 0, 1, 1, 0);
+	let _: &'static BitSlice = bits!(static 1, 0, 1, 1, 0);
+}
+
 #[test]
 fn compile_bitarr() {
 	let uint: BitArray<Lsb0, [u8; 1]> = bitarr![Lsb0, u8; 1, 0, 1, 0];
-	assert_eq!(uint.value(), [5u8]);
+	assert_eq!(uint.into_inner(), [5u8]);
 	let cell: BitArray<Lsb0, [Cell<u8>; 1]> =
 		bitarr![Lsb0, Cell<u8>; 1, 0, 1, 0];
-	assert_eq!(cell.value()[0].get(), 5u8);
+	assert_eq!(cell.into_inner()[0].get(), 5u8);
 
 	let uint: BitArray<Msb0, [u16; 2]> = bitarr![Msb0, u16;
 		0, 1, 0, 1, 0, 1, 0, 1,
@@ -40,43 +97,43 @@ fn compile_bitarr() {
 		0, 1, 1, 0, 1, 1, 1, 0,
 		0, 1, 1, 1, 0, 1, 0, 0,
 	];
-	assert_eq!(uint.value(), [0x5569, 0x6e74]);
+	assert_eq!(uint.into_inner(), [0x5569, 0x6e74]);
 	let cell: BitArray<Msb0, [Cell<u16>; 2]> = bitarr![Msb0, Cell<u16>;
 		0, 1, 0, 1, 0, 1, 0, 1,
 		0, 1, 1, 0, 1, 0, 0, 1,
 		0, 1, 1, 0, 1, 1, 1, 0,
 		0, 1, 1, 1, 0, 1, 0, 0,
 	];
-	let cells = cell.value();
+	let cells = cell.into_inner();
 	assert_eq!(cells[0].get(), 0x5569);
 	assert_eq!(cells[1].get(), 0x6e74);
 
 	let uint: BitArray<Lsb0, [u32; 1]> = bitarr![crate::order::Lsb0, u32;
 		1, 0, 1, 1,
 	];
-	assert_eq!(uint.value(), [13u32]);
+	assert_eq!(uint.into_inner(), [13u32]);
 	let cell: BitArray<Lsb0, [Cell<u32>; 1]> = bitarr![
 		crate::order::Lsb0, Cell<u32>;
 		1, 0, 1, 1,
 	];
-	assert_eq!(cell.value()[0].get(), 13u32);
+	assert_eq!(cell.into_inner()[0].get(), 13u32);
 
 	#[cfg(target_pointer_width = "64")]
 	{
 		let uint: BitArray<LocalBits, [u64; 2]> = bitarr![LocalBits, u64; 1; 70];
-		assert_eq!(uint.value(), [!0u64; 2]);
+		assert_eq!(uint.into_inner(), [!0u64; 2]);
 
 		let cell: BitArray<LocalBits, [Cell<u64>; 2]> = bitarr![
 			LocalBits, Cell<u64>; 1; 70
 		];
-		assert_eq!(cell.clone().value()[0].get(), !0u64);
-		assert_eq!(cell.value()[1].get(), !0u64);
+		assert_eq!(cell.clone().into_inner()[0].get(), !0u64);
+		assert_eq!(cell.into_inner()[1].get(), !0u64);
 	}
 
 	let uint: BitArray<Lsb0, [usize; 1]> = bitarr![1, 0, 1];
-	assert_eq!(uint.value(), [5usize]);
+	assert_eq!(uint.into_inner(), [5usize]);
 	let uint: BitArray<Lsb0, [usize; 1]> = bitarr![1; 30];
-	assert_eq!(uint.value(), [!0usize]);
+	assert_eq!(uint.into_inner(), [!0usize]);
 }
 
 #[test]
@@ -96,7 +153,7 @@ fn compile_bits() {
 	let h: &mut BitSlice<Msb0, u8> = bits![mut crate::order::Msb0, u8; 1; 100];
 	assert_eq!(e, g);
 	assert_eq!(f, h);
-	assert_eq!(h.as_slice(), [!0u8; 13]);
+	assert_eq!(h.as_raw_slice(), [!0u8; 13]);
 
 	let i: &mut BitSlice<Lsb0, usize> = bits![mut 1, 0, 1];
 	let j: &mut BitSlice<Lsb0, usize> = bits![mut 1; 3];
@@ -511,13 +568,12 @@ fn make_elem() {
 	let bits: &BitSlice<LocalBits, Cell<u32>> = cell.view_bits::<_>();
 	assert_eq!(bits[.. 3], bits![1, 0, 1]);
 
-	//  `__make_elem!` is only called after `$ord` has already been made opaque
-	//  to matchers as a single `:tt`. Calling it directly with a path will fail
-	//  the `:tt`, so this macro wraps it as one and forwards the rest.
+	//  `__make_elem!` is only called after `$ord` has already been made
+	// opaque  to matchers as a single `:tt`. Calling it directly with a path
+	// will fail  the `:tt`, so this macro wraps it as one and forwards the
+	// rest.
 	macro_rules! invoke_make_elem {
-		($ord:path, $($rest:tt)*) => {
-			__make_elem!($ord, $($rest)*)
-		};
+		($ord:path, $($rest:tt)*) => { __make_elem!($ord, $($rest)*) };
 	}
 	let uint: usize =
 		invoke_make_elem!(crate::order::Lsb0, usize as usize; 0, 0, 1, 1);
